"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const web3_1 = require("./web3");
const tezos_1 = require("./tezos");
const getNetworkTypeClass = (networkType = "ethereum") => {
    const supportedEvmNetworks = ["ethereum", "fabric-evm", "quorum", "web3js"];
    if (supportedEvmNetworks.includes(networkType))
        return "evm-like";
    return networkType;
};
exports.createInterfaceAdapter = (options) => {
    const { provider, networkType } = options;
    switch (getNetworkTypeClass(networkType)) {
        case "evm-like": {
            return new web3_1.Web3InterfaceAdapter({
                networkType,
                provider
            });
        }
        case "tezos": {
            return new tezos_1.TezosAdapter({
                provider
            });
        }
        default:
            throw Error(`Sorry, "${networkType}" is not supported at this time.`);
    }
};
//# sourceMappingURL=index.js.map